/*
 * The Unified Mapping Platform (JUMP) is an extensible, interactive GUI 
 * for visualizing and manipulating spatial features with geometry and attributes.
 * 
 * Code is based on code from com.vividsolutions.jump.workbench.ui.plugin.wms.AddWMSQueryPlugIn.java
 * and on code from e-mail 
 * Message-ID: <5369d92b041105233553cb5ae8@mail.gmail.com>
 * Date: Fri, 5 Nov 2004 23:35:15 -0800
 * From: Jonathan Aquino <jonathan.aquino@gmail.com>
 * To: JUMP Users Discussion <jump-users@lists.jump-project.org>
 * Subject: [jump-users] BeanShell to create an image layer from a file or URL	!!! gif/jpeg/png
 *
 * $Id: AddImagePlugIn.java,v 0.1 20041110 
 *
 * Copyright (C) 2004 Jan Ruzicka jan.ruzicka@vsb.cz
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

/**
 *
 * <p>Title: AddImagePlugIn</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author Jan Ruzicka jan.ruzicka@vsb.cz
 * @version 0.1
 */

package cz.vsb.gisak;


import com.vividsolutions.jump.workbench.model.StandardCategoryNames;
import com.vividsolutions.jump.workbench.plugin.AbstractPlugIn;
import com.vividsolutions.jump.workbench.plugin.PlugInContext;
import com.vividsolutions.jump.workbench.ui.GUIUtil;
import com.vividsolutions.jump.workbench.ui.wizard.WizardDialog;
import com.vividsolutions.jump.workbench.ui.wizard.WizardPanel;

import com.vividsolutions.jts.geom.*;
import com.vividsolutions.jump.feature.*;
import com.vividsolutions.jump.geom.EnvelopeUtil;
import com.vividsolutions.jump.workbench.WorkbenchContext;


import java.awt.*;
import java.util.*;
import java.io.*;



public class AddImagePlugIn extends AbstractPlugIn {
	private Image image;
	private final String sep = System.getProperty("file.separator");

    private String cachedFile = "Type file name with path or use browse";
    private String cachedMinx = "Type West coordinate";
    private String cachedMaxx = "Type East coordinate";
    private String cachedMiny = "Type South coordinate";
    private String cachedMaxy = "Type North coordinate";
    private String cachedLayer = "Image";

    public AddImagePlugIn() {

    }

    public void setImage(Image image){
    	this.image = image;
    }
    
    
    private boolean addImage(WorkbenchContext context, String url, double cminx, double cmaxx, double cminy, double cmaxy) {
    	final double minx, maxx, miny, maxy;
    	String filenameOrURL = "";
    	String layerName;
       	final Envelope envelope;
    	minx = cminx; maxx = cmaxx; miny = cminy; maxy = cmaxy; //Image coordinates
    	filenameOrURL = url;
    	layerName = cachedLayer;
    		
	envelope = new Envelope(minx, maxx, miny, maxy);
	
	StyleImage s = new StyleImage(envelope,image,minx,maxy);
	s.setFilePath(cachedFile);
	context.getLayerManager().addLayer(StandardCategoryNames.WORKING, layerName, 
	FeatureDatasetFactory.createFromGeometry(Collections.singleton(EnvelopeUtil.toGeometry(envelope)))).addStyle(s);
	context.getLayerManager().getLayer(layerName).getBasicStyle().setRenderingFill(false);
	return(true);	
    }

    public void initialize(PlugInContext context) throws Exception {
        context.getFeatureInstaller().addMainMenuItem(this,
        new String[] { "Layer", "Add New Image Layer" }, getName(), false, null, null);
    }


    public boolean execute(final PlugInContext context)
        throws Exception {
        reportNothingToUndoYet(context);
        WizardDialog d = new WizardDialog(context.getWorkbenchFrame(),
                "Add Image to Task", context.getErrorHandler());
        d.init(new WizardPanel[] {
                new ImageWizardPanel(cachedFile, cachedMinx, cachedMaxx, cachedMiny, cachedMaxy, cachedLayer, this)
            });

        //Set size after #init, because #init calls #pack. [Jon Aquino]
        d.setSize(500, 400);
        GUIUtil.centreOnWindow(d);
        d.setVisible(true);
        if (!d.wasFinishPressed()) {
            return false;
        }

        cachedFile = (String) d.getData(ImageWizardPanel.FILE_KEY);
        cachedMinx = (String) d.getData(ImageWizardPanel.MINX_KEY);
        cachedMaxx = (String) d.getData(ImageWizardPanel.MAXX_KEY);
        cachedMiny = (String) d.getData(ImageWizardPanel.MINY_KEY);
        cachedMaxy = (String) d.getData(ImageWizardPanel.MAXY_KEY);
        cachedLayer = (String) d.getData(ImageWizardPanel.LAYER_KEY);


	double minx, maxx, miny, maxy;
	minx = Double.parseDouble((String) d.getData(ImageWizardPanel.MINX_KEY));
	maxx = Double.parseDouble((String) d.getData(ImageWizardPanel.MAXX_KEY));
	miny = Double.parseDouble((String) d.getData(ImageWizardPanel.MINY_KEY));
	maxy = Double.parseDouble((String) d.getData(ImageWizardPanel.MAXY_KEY));

	addImage(context.getWorkbenchContext(), cachedFile, minx, maxx, miny, maxy);
        
        return true;
    }
}
