/*
 * The Unified Mapping Platform (JUMP) is an extensible, interactive GUI 
 * for visualizing and manipulating spatial features with geometry and attributes.
 * 
 * Code is based on code from com.vividsolutions.jump.workbench.ui.plugin.wms.URLWizardPanel.java
 *
 * $Id: ImageWizardPanel.java,v 0.1 20041110 
 *
 * Copyright (C) 2004 Jan Ruzicka jan.ruzicka@vsb.cz
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 * 
 */

/**
 *
 * <p>Title: ImageWizardPanel</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2004</p>
 * <p>Company: </p>
 * @author Jan Ruzicka jan.ruzicka@vsb.cz
 * @version 0.1
 */

package cz.vsb.gisak;

import java.awt.Dimension;
import java.awt.Image;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.*;

import java.util.Map;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.SwingUtilities;
import javax.swing.JFileChooser;


import com.vividsolutions.jump.workbench.WorkbenchException;
import com.vividsolutions.jump.workbench.ui.InputChangedFirer;
import com.vividsolutions.jump.workbench.ui.InputChangedListener;
import com.vividsolutions.jump.workbench.ui.wizard.WizardPanel;

/**
* Panel for setting properties for viewed image.
* @author jan.ruzicka@vsb.cz
*/

public class ImageWizardPanel extends JPanel implements WizardPanel, ActionListener {
	public static AddImagePlugIn aip;
	public static final String FILE_KEY = "FILE";
    public static String MINX_KEY = "MINX";
    public static String MAXX_KEY = "MAXX";
    public static String MINY_KEY = "MINY";
    public static String MAXY_KEY = "MAXY";
    public static final String LAYER_KEY = "LAYER";
    private InputChangedFirer inputChangedFirer = new InputChangedFirer();
    private Map dataMap;
    private GridBagLayout gridBagLayout1 = new GridBagLayout();
    private JLabel fileLabel = new JLabel();
    private JTextField fileTextField = new JTextField();
    private JButton fileButton = new JButton("Browse");
    private JPanel fillerPanel = new JPanel();
    private JLabel minxLabel = new JLabel();
    private JTextField minxTextField = new JTextField();
    private JLabel maxxLabel = new JLabel();
    private JTextField maxxTextField = new JTextField();
    private JLabel minyLabel = new JLabel();
    private JTextField minyTextField = new JTextField();
    private JLabel maxyLabel = new JLabel();
    private JTextField maxyTextField = new JTextField();
    private JLabel layerLabel = new JLabel();
    private JTextField layerTextField = new JTextField();
   

    public ImageWizardPanel(String initialFile, String initialMinx, String initialMaxx, String initialMiny, String initialMaxy, String initialLayer, AddImagePlugIn a) {
        try {
            jbInit();
            fileTextField.setText(initialFile);
            minxTextField.setText(initialMinx);
            maxxTextField.setText(initialMaxx);
            minyTextField.setText(initialMiny);
            maxyTextField.setText(initialMaxy);
            layerTextField.setText(initialLayer);
            aip=a;

        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void add(InputChangedListener listener) {
        inputChangedFirer.add(listener);
    }

    public void remove(InputChangedListener listener) {
        inputChangedFirer.remove(listener);
    }
   
    /**
    * Creates UI on JPanel
    */
    void jbInit() throws Exception {
        fileLabel.setText("File Name:");
        minxLabel.setText("MinX:");
	maxxLabel.setText("MaxX:");
	minyLabel.setText("MinY:");
	maxyLabel.setText("MaxY:");
        layerLabel.setText("Layer name:");
        this.setLayout(gridBagLayout1);
        fileTextField.setPreferredSize(new Dimension(300, 21));
        fileTextField.setCaretPosition(fileTextField.getText().length());
	fileButton.setPreferredSize(new Dimension(30, 21));
        minxTextField.setPreferredSize(new Dimension(300, 21));
        minxTextField.setCaretPosition(minxTextField.getText().length());
 	maxxTextField.setPreferredSize(new Dimension(300, 21));
        maxxTextField.setCaretPosition(maxxTextField.getText().length());
	minyTextField.setPreferredSize(new Dimension(300, 21));
        minyTextField.setCaretPosition(minyTextField.getText().length());
	maxyTextField.setPreferredSize(new Dimension(300, 21));
        maxyTextField.setCaretPosition(maxyTextField.getText().length());
	layerTextField.setPreferredSize(new Dimension(300, 21));
        layerTextField.setCaretPosition(layerTextField.getText().length());

       	fileButton.addActionListener(this);

        this.add(fileLabel,
            new GridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
                GridBagConstraints.CENTER, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 4), 0, 0));
        this.add(fileTextField,
            new GridBagConstraints(1, 1, 1, 1, 0.0, 0.0,
                GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
                new Insets(0, 0, 0, 4), 0, 0));
 	this.add(fileButton,
            new GridBagConstraints(2, 1, 1, 1, 0.0, 0.0,
                GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
                new Insets(0, 0, 0, 4), 0, 0));
        this.add(minxLabel,
            new GridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
                GridBagConstraints.CENTER, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 4), 0, 0));
        this.add(minxTextField,
            new GridBagConstraints(1, 2, 1, 1, 0.0, 0.0,
                GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
                new Insets(0, 0, 0, 4), 0, 0));
  	this.add(maxxLabel,
            new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
                GridBagConstraints.CENTER, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 4), 0, 0));
        this.add(maxxTextField,
            new GridBagConstraints(1, 3, 1, 1, 0.0, 0.0,
                GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
                new Insets(0, 0, 0, 4), 0, 0));
	this.add(minyLabel,
            new GridBagConstraints(0, 4, 1, 1, 0.0, 0.0,
                GridBagConstraints.CENTER, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 4), 0, 0));
        this.add(minyTextField,
            new GridBagConstraints(1, 4, 1, 1, 0.0, 0.0,
                GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
                new Insets(0, 0, 0, 4), 0, 0));
	this.add(maxyLabel,
            new GridBagConstraints(0, 5, 1, 1, 0.0, 0.0,
                GridBagConstraints.CENTER, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 4), 0, 0));
        this.add(maxyTextField,
            new GridBagConstraints(1, 5, 1, 1, 0.0, 0.0,
                GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
                new Insets(0, 0, 0, 4), 0, 0));
	this.add(layerLabel,
            new GridBagConstraints(0, 6, 1, 1, 0.0, 0.0,
                GridBagConstraints.CENTER, GridBagConstraints.NONE,
                new Insets(0, 0, 0, 4), 0, 0));
        this.add(layerTextField,
            new GridBagConstraints(1, 6, 1, 1, 0.0, 0.0,
                GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL,
                new Insets(0, 0, 0, 4), 0, 0));

        this.add(fillerPanel,
            new GridBagConstraints(2, 10, 1, 1, 1.0, 1.0,
                GridBagConstraints.CENTER, GridBagConstraints.BOTH,
                new Insets(0, 0, 0, 0), 0, 0));
	
    }

    public String getInstructions() {
        return "Please enter the image path (this plugin accepts PNG, GIF and JPEG formats) and Extent. Then press Finish to add image to the Task.";
    }

    /**
    * Cuts spaces from given pathname
    */
    private String fix(String file) { 
   	return file.trim();
    }

    /**
    * Works after Finish button. Sets the values from text boxes to the dataMap object.
    */
    public void exitingToRight() throws IOException, WorkbenchException {
    dataMap.put(FILE_KEY, fix(fileTextField.getText()));
	dataMap.put(MINX_KEY, minxTextField.getText());
	dataMap.put(MAXX_KEY, maxxTextField.getText());
	dataMap.put(MINY_KEY, minyTextField.getText());
	dataMap.put(MAXY_KEY, maxyTextField.getText());
	dataMap.put(LAYER_KEY, layerTextField.getText());
    }

    public void enteredFromLeft(Map dataMap) {
        this.dataMap = dataMap;
    }

    public String getTitle() {
        return "Select (set) the file name and path and set the extent";
    }

    public String getID() {
        return getClass().getName();
    }

    public boolean isInputValid() {
        return true;
    }

    public String getNextID() {
        return null;
    }

    /**
    * Works after Browse button. Opens OpenFile Dialog.
    */
    public void actionPerformed(ActionEvent e) {
    double param[] = new double[7];
	JFileChooser chooser = new JFileChooser();
	int returnVal = chooser.showOpenDialog(this);
    String path;
        if(returnVal == JFileChooser.APPROVE_OPTION) {
               path = chooser.getSelectedFile().getPath(); 
	       System.out.println("You chose to open this file: " + path);
	       fileTextField.setText(path);
	      	Image image;
	    	image = Toolkit.getDefaultToolkit().getImage(path);
	     	MediaTracker mediaTracker;
	    	mediaTracker = new MediaTracker(this);
	    	mediaTracker.addImage(image, 0);
	    	try {
	    		mediaTracker.waitForID(0);
	    	} catch (java.lang.InterruptedException ie) {
	    	System.out.println("Problems with drawing image in Panel: " + ie);
	    	}
	    	aip.setImage(image);
	    	
	    	
           String sufix = path.substring(path.length()-3, path.length()).toLowerCase();
           String prefix = path.substring(0, path.length()-3);
           String georefPath="";
           if(sufix.equals("png")) georefPath=prefix+"pgw";
           if(sufix.equals("bmp")) georefPath=prefix+"bpw";
           if(sufix.equals("jpg")) georefPath=prefix+"jgw";
           if(sufix.equals("tif")) georefPath=prefix+"tfw";
	       try { 
	       	    File fil = new File (georefPath);
	            if (fil.exists()){
	                    FileReader fr = new java.io.FileReader(fil); 
	                    BufferedReader bfr = new java.io.BufferedReader(fr);
	                    String ln;
	                    byte i=0;
	                    while ((ln = bfr.readLine()) != null) {
	                        i++;
	                        param[i] =Double.parseDouble(ln);
	                    }
	            bfr.close();
	            fr.close();
	            }
	       }  catch (IOException e1)  {
	                System.out.println("File read error...");
	       }

           minxTextField.setText(param[5]+"");
           maxxTextField.setText((param[5]+param[1]*image.getWidth(null))+"");
           minyTextField.setText((param[6]+param[4]*image.getHeight(null))+"");
           maxyTextField.setText(param[6]+"");
        }	
    }
}

